@@ -1219,7 +1219,7 @@ FixSegmentHeader(PNET_BUFFER nb, UINT16 segmentSize, UINT32 seqNumber,
 
 /*
  * --------------------------------------------------------------------------
- * OvsTcpSegmentyNBL --
+ * OvsTcpSegmentNBL --
  *
  *    Segment TCP payload, and prepend each segment with ether/IP/TCP header.
  *    Leave headRoom for additional encap.
@@ -1477,7 +1477,6 @@ OvsCompleteNBL(POVS_SWITCH_CONTEXT context,
 {
     POVS_BUFFER_CONTEXT ctx;
     UINT16 flags;
-    UINT32 dataOffsetDelta;
     PNET_BUFFER_LIST parent;
     NDIS_STATUS status;
     NDIS_HANDLE poolHandle;
@@ -1508,7 +1507,6 @@ OvsCompleteNBL(POVS_SWITCH_CONTEXT context,
     nb = NET_BUFFER_LIST_FIRST_NB(nbl);
 
     flags = ctx->flags;
-    dataOffsetDelta = ctx->dataOffsetDelta;
     if (!(flags & OVS_BUFFER_FRAGMENT) &&
         NET_BUFFER_DATA_LENGTH(nb) != ctx->origDataLength) {
         UINT32 diff;
@@ -1523,7 +1521,7 @@ OvsCompleteNBL(POVS_SWITCH_CONTEXT context,
         }
     }
 
-    if (flags & OVS_BUFFER_PRIVATE_CONTEXT) {
+    if (ctx->flags & OVS_BUFFER_PRIVATE_CONTEXT) {
         NdisFreeNetBufferListContext(nbl, sizeof (OVS_BUFFER_CONTEXT));
     }
 
@@ -1543,27 +1541,27 @@ OvsCompleteNBL(POVS_SWITCH_CONTEXT context,
     }
 
     if (flags & (OVS_BUFFER_PRIVATE_MDL | OVS_BUFFER_PRIVATE_DATA)) {
-        PNET_BUFFER nb = NET_BUFFER_LIST_FIRST_NB(nbl);
-        while (nb) {
-            PMDL mdl = NET_BUFFER_FIRST_MDL(nb);
-            NET_BUFFER_FIRST_MDL(nb) = NULL;
+        PNET_BUFFER nbTemp = NET_BUFFER_LIST_FIRST_NB(nbl);
+        while (nbTemp) {
+            PMDL mdl = NET_BUFFER_FIRST_MDL(nbTemp);
+            NET_BUFFER_FIRST_MDL(nbTemp) = NULL;
             ASSERT(mdl->Next == NULL);
             OvsFreeMDLAndData(mdl);
-            nb = NET_BUFFER_NEXT_NB(nb);
+            nbTemp = NET_BUFFER_NEXT_NB(nbTemp);
         }
     }
 
     if (flags & OVS_BUFFER_PRIVATE_NET_BUFFER) {
-        PNET_BUFFER nb, nextNb;
+        PNET_BUFFER nbTemp, nextNb;
 
-        nb = NET_BUFFER_LIST_FIRST_NB(nbl);
-        while (nb) {
-            nextNb = NET_BUFFER_NEXT_NB(nb);
-            NdisFreeNetBuffer(nb);
+        nbTemp = NET_BUFFER_LIST_FIRST_NB(nbl);
+        while (nbTemp) {
+            nextNb = NET_BUFFER_NEXT_NB(nbTemp);
+            NdisFreeNetBuffer(nbTemp);
 #ifdef DBG
             InterlockedDecrement((LONG volatile *)&ovsPool->nbCount);
 #endif
-            nb = nextNb;
+            nbTemp = nextNb;
         }
         NET_BUFFER_LIST_FIRST_NB(nbl) = NULL;
     }
@@ -1594,7 +1592,7 @@ OvsCompleteNBL(POVS_SWITCH_CONTEXT context,
 #ifdef DBG
         InterlockedDecrement((LONG volatile *)&ovsPool->fragNBLCount);
 #endif
-        NdisFreeFragmentNetBufferList(nbl, dataOffsetDelta, 0);
+        NdisFreeFragmentNetBufferList(nbl, ctx->dataOffsetDelta, 0);
     }
 
     if (parent != NULL) {
