@@ -156,7 +156,7 @@ struct dpif_class {
     int (*get_stats)(const struct dpif *dpif, struct dpif_dp_stats *stats);
 
     /* Adds 'netdev' as a new port in 'dpif'.  If '*port_no' is not
-     * UINT32_MAX, attempts to use that as the port's port number.
+     * ODPP_NONE, attempts to use that as the port's port number.
      *
      * If port is successfully added, sets '*port_no' to the new port's
      * port number.  Returns EBUSY if caller attempted to choose a port
@@ -167,6 +167,11 @@ struct dpif_class {
     /* Removes port numbered 'port_no' from 'dpif'. */
     int (*port_del)(struct dpif *dpif, odp_port_t port_no);
 
+    /* Refreshes configuration of 'dpif's port. The implementation might
+     * postpone applying the changes until run() is called. */
+    int (*port_set_config)(struct dpif *dpif, odp_port_t port_no,
+                           const struct smap *cfg);
+
     /* Queries 'dpif' for a port with the given 'port_no' or 'devname'.
      * If 'port' is not null, stores information about the port into
      * '*port' if successful.
@@ -318,11 +323,10 @@ struct dpif_class {
     int (*handlers_set)(struct dpif *dpif, uint32_t n_handlers);
 
     /* If 'dpif' creates its own I/O polling threads, refreshes poll threads
-     * configuration.  'n_rxqs' configures the number of rx_queues, which
-     * are distributed among threads.  'cmask' configures the cpu mask
-     * for setting the polling threads' cpu affinity. */
-    int (*poll_threads_set)(struct dpif *dpif, unsigned int n_rxqs,
-                            const char *cmask);
+     * configuration.  'cmask' configures the cpu mask for setting the polling
+     * threads' cpu affinity.  The implementation might postpone applying the
+     * changes until run() is called. */
+    int (*poll_threads_set)(struct dpif *dpif, const char *cmask);
 
     /* Translates OpenFlow queue ID 'queue_id' (in host byte order) into a
      * priority value used for setting packet priority. */
@@ -397,7 +401,7 @@ struct dpif_class {
     /* Conntrack entry dumping interface.
      *
      * These functions are used by ct-dpif.c to provide a datapath-agnostic
-     * dumping interface to the connection trackes provided by the
+     * dumping interface to the connection trackers provided by the
      * datapaths.
      *
      * ct_dump_start() should put in '*state' a pointer to a newly allocated
@@ -408,11 +412,11 @@ struct dpif_class {
      * ct_dump_next() should fill 'entry' with information from a connection
      * and prepare to dump the next one on a subsequest invocation.
      *
-     * ct_dump_done should perform any cleanup necessary (including
+     * ct_dump_done() should perform any cleanup necessary (including
      * deallocating the 'state' structure, if applicable). */
     int (*ct_dump_start)(struct dpif *, struct ct_dpif_dump_state **state,
                          const uint16_t *zone);
-    int (*ct_dump_next)(struct dpif *, struct ct_dpif_dump_state *,
+    int (*ct_dump_next)(struct dpif *, struct ct_dpif_dump_state *state,
                         struct ct_dpif_entry *entry);
     int (*ct_dump_done)(struct dpif *, struct ct_dpif_dump_state *state);
 
