@@ -661,7 +661,7 @@ miniflow_extract(struct dp_packet *packet, struct miniflow *dst)
         if (OVS_UNLIKELY(tot_len > size || ip_len > tot_len)) {
             goto out;
         }
-        if (OVS_UNLIKELY(size - tot_len > UINT16_MAX)) {
+        if (OVS_UNLIKELY(size - tot_len > UINT8_MAX)) {
             goto out;
         }
         dp_packet_set_l2_pad_size(packet, size - tot_len);
@@ -696,7 +696,8 @@ miniflow_extract(struct dp_packet *packet, struct miniflow *dst)
         if (OVS_UNLIKELY(plen > size)) {
             goto out;
         }
-        if (OVS_UNLIKELY(size - plen > UINT16_MAX)) {
+        /* Jumbo Payload option not supported yet. */
+        if (OVS_UNLIKELY(size - plen > UINT8_MAX)) {
             goto out;
         }
         dp_packet_set_l2_pad_size(packet, size - plen);
@@ -708,6 +709,11 @@ miniflow_extract(struct dp_packet *packet, struct miniflow *dst)
                             sizeof nh->ip6_dst / 8);
 
         tc_flow = get_16aligned_be32(&nh->ip6_flow);
+        {
+            ovs_be32 label = tc_flow & htonl(IPV6_LABEL_MASK);
+            miniflow_push_be32(mf, ipv6_label, label);
+        }
+
         nw_tos = ntohl(tc_flow) >> 20;
         nw_ttl = nh->ip6_hlim;
         nw_proto = nh->ip6_nxt;
@@ -715,12 +721,6 @@ miniflow_extract(struct dp_packet *packet, struct miniflow *dst)
         if (!parse_ipv6_ext_hdrs__(&data, &size, &nw_proto, &nw_frag)) {
             goto out;
         }
-
-        /* This needs to be after the parse_ipv6_ext_hdrs__() call because it
-         * leaves the nw_frag word uninitialized. */
-        ASSERT_SEQUENTIAL(ipv6_label, nw_frag);
-        ovs_be32 label = tc_flow & htonl(IPV6_LABEL_MASK);
-        miniflow_push_be32(mf, ipv6_label, label);
     } else {
         if (dl_type == htons(ETH_TYPE_ARP) ||
             dl_type == htons(ETH_TYPE_RARP)) {
@@ -1298,6 +1298,7 @@ void flow_wildcards_init_for_packet(struct flow_wildcards *wc,
                 wc->masks.tunnel.metadata.present.map =
                                               flow->tunnel.metadata.present.map;
                 WC_MASK_FIELD(wc, tunnel.metadata.opts.u8);
+                WC_MASK_FIELD(wc, tunnel.metadata.tab);
             }
         } else {
             WC_MASK_FIELD(wc, tunnel.metadata.present.len);
@@ -1769,7 +1770,7 @@ flow_hash_symmetric_l3l4(const struct flow *flow, uint32_t basis,
         const uint64_t *a = ALIGNED_CAST(uint64_t *, flow->ipv6_src.s6_addr);
         const uint64_t *b = ALIGNED_CAST(uint64_t *, flow->ipv6_dst.s6_addr);
 
-        for (int i = 0; i < sizeof flow->ipv6_src / sizeof *a; i++) {
+        for (int i = 0; i < 4; i++) {
             hash = hash_add64(hash, a[i] ^ b[i]);
         }
     } else {
