@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2016, 2017 Nicira, Inc.
+ * Copyright (c) 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2016 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -332,14 +332,6 @@ netdev_open(const char *name, const char *type, struct netdev **netdevp)
     struct netdev *netdev;
     int error;
 
-    if (!name[0]) {
-        /* Reject empty names.  This saves the providers having to do this.  At
-         * least one screwed this up: the netdev-linux "tap" implementation
-         * passed the name directly to the Linux TUNSETIFF call, which treats
-         * an empty string as a request to generate a unique name. */
-        return EINVAL;
-    }
-
     netdev_initialize();
 
     ovs_mutex_lock(&netdev_mutex);
@@ -829,13 +821,7 @@ netdev_set_etheraddr(struct netdev *netdev, const struct eth_addr mac)
 int
 netdev_get_etheraddr(const struct netdev *netdev, struct eth_addr *mac)
 {
-    int error;
-
-    error = netdev->netdev_class->get_etheraddr(netdev, mac);
-    if (error) {
-        memset(mac, 0, sizeof *mac);
-    }
-    return error;
+    return netdev->netdev_class->get_etheraddr(netdev, mac);
 }
 
 /* Returns the name of the network device that 'netdev' represents,
@@ -1650,9 +1636,8 @@ bool
 netdev_queue_dump_next(struct netdev_queue_dump *dump,
                        unsigned int *queue_id, struct smap *details)
 {
-    smap_clear(details);
-
     const struct netdev *netdev = dump->netdev;
+
     if (dump->error) {
         return false;
     }
@@ -1781,8 +1766,8 @@ netdev_get_vports(size_t *size)
     }
 
     /* Explicitly allocates big enough chunk of memory. */
-    vports = xmalloc(shash_count(&netdev_shash) * sizeof *vports);
     ovs_mutex_lock(&netdev_mutex);
+    vports = xmalloc(shash_count(&netdev_shash) * sizeof *vports);
     SHASH_FOR_EACH (node, &netdev_shash) {
         struct netdev *dev = node->data;
 
@@ -1897,7 +1882,7 @@ netdev_get_addrs(const char dev[], struct in6_addr **paddr,
     }
 
     for (ifa = if_addr_list; ifa; ifa = ifa->ifa_next) {
-        if (ifa->ifa_addr && ifa->ifa_name && ifa->ifa_netmask) {
+        if (ifa->ifa_addr != NULL) {
             int family;
 
             family = ifa->ifa_addr->sa_family;
@@ -1918,8 +1903,7 @@ netdev_get_addrs(const char dev[], struct in6_addr **paddr,
     for (ifa = if_addr_list; ifa; ifa = ifa->ifa_next) {
         int family;
 
-        if (!ifa->ifa_name || !ifa->ifa_addr || !ifa->ifa_netmask
-            || strncmp(ifa->ifa_name, dev, IFNAMSIZ)) {
+        if (strncmp(ifa->ifa_name, dev, IFNAMSIZ) || ifa->ifa_addr == NULL) {
             continue;
         }
 
