@@ -50,13 +50,18 @@ char *mf_parse_subfield(struct mf_subfield *, const char *s)
 /* Decoding matches. */
 enum ofperr nx_pull_match(struct ofpbuf *, unsigned int match_len,
                           struct match *,
-                          ovs_be64 *cookie, ovs_be64 *cookie_mask);
+                          ovs_be64 *cookie, ovs_be64 *cookie_mask,
+                          const struct tun_table *);
 enum ofperr nx_pull_match_loose(struct ofpbuf *, unsigned int match_len,
                                 struct match *, ovs_be64 *cookie,
-                                ovs_be64 *cookie_mask);
-enum ofperr oxm_pull_match(struct ofpbuf *, struct match *);
-enum ofperr oxm_pull_match_loose(struct ofpbuf *, struct match *);
-enum ofperr oxm_decode_match(const void *, size_t, struct match *);
+                                ovs_be64 *cookie_mask,
+                                const struct tun_table *);
+enum ofperr oxm_pull_match(struct ofpbuf *, const struct tun_table *,
+                           struct match *);
+enum ofperr oxm_pull_match_loose(struct ofpbuf *, const struct tun_table *,
+                                 struct match *);
+enum ofperr oxm_decode_match(const void *, size_t, const struct tun_table *,
+                             struct match *);
 enum ofperr oxm_pull_field_array(const void *, size_t fields_len,
                                  struct field_array *);
 
