@@ -142,11 +142,13 @@ eth_addr_is_reserved(const struct eth_addr ea)
 
 /* Attempts to parse 's' as an Ethernet address.  If successful, stores the
  * address in 'ea' and returns true, otherwise zeros 'ea' and returns
- * false.  */
+ * false.  This function checks trailing characters. */
 bool
 eth_addr_from_string(const char *s, struct eth_addr *ea)
 {
-    if (ovs_scan(s, ETH_ADDR_SCAN_FMT, ETH_ADDR_SCAN_ARGS(*ea))) {
+    int n = 0;
+    if (ovs_scan(s, ETH_ADDR_SCAN_FMT"%n", ETH_ADDR_SCAN_ARGS(*ea), &n)
+        && !s[n]) {
         return true;
     } else {
         *ea = eth_addr_zero;
@@ -427,6 +429,24 @@ ip_parse(const char *s, ovs_be32 *ip)
     return inet_pton(AF_INET, s, ip) == 1;
 }
 
+/* Parses string 's', which must be an IP address with a port number
+ * with ":" as a separator (e.g.: 192.168.1.2:80).
+ * Stores the IP address into '*ip' and port number to '*port'.
+ *
+ * Returns NULL if successful, otherwise an error message that the caller must
+ * free(). */
+char * OVS_WARN_UNUSED_RESULT
+ip_parse_port(const char *s, ovs_be32 *ip, ovs_be16 *port)
+{
+    int n = 0;
+    if (ovs_scan(s, IP_PORT_SCAN_FMT"%n", IP_PORT_SCAN_ARGS(ip, port), &n)
+        && !s[n]) {
+        return NULL;
+    }
+
+    return xasprintf("%s: invalid IP address or port number", s);
+}
+
 /* Parses string 's', which must be an IP address with an optional netmask or
  * CIDR prefix length.  Stores the IP address into '*ip', netmask into '*mask',
  * (255.255.255.255, if 's' lacks a netmask), and number of scanned characters
@@ -1454,26 +1474,6 @@ packet_csum_pseudoheader6(const struct ovs_16aligned_ip6_hdr *ip6)
 
     return partial;
 }
-
-/* Calculate the IPv6 upper layer checksum according to RFC2460. We pass the
-   ip6_nxt and ip6_plen values, so it will also work if extension headers
-   are present. */
-ovs_be16
-packet_csum_upperlayer6(const struct ovs_16aligned_ip6_hdr *ip6,
-                        const void *data, uint8_t l4_protocol,
-                        uint16_t l4_size)
-{
-    uint32_t partial = 0;
-
-    partial = csum_continue(partial, &ip6->ip6_src, sizeof ip6->ip6_src);
-    partial = csum_continue(partial, &ip6->ip6_dst, sizeof ip6->ip6_dst);
-    partial = csum_add16(partial, htons(l4_protocol));
-    partial = csum_add16(partial, htons(l4_size));
-
-    partial = csum_continue(partial, data, l4_size);
-
-    return csum_finish(partial);
-}
 #endif
 
 void
