@@ -1,4 +1,4 @@
-/* Copyright (c) 2015 Nicira, Inc.
+/* Copyright (c) 2015, 2016 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -25,9 +25,9 @@
 struct controller_ctx;
 struct hmap;
 struct ovsrec_bridge;
+struct sbrec_chassis;
 
 void patch_run(struct controller_ctx *, const struct ovsrec_bridge *br_int,
-               const char *chassis_id, struct hmap *local_datapaths,
-               struct hmap *patched_datapaths);
+               const struct sbrec_chassis *, struct hmap *local_datapaths);
 
 #endif /* ovn/patch.h */
