@@ -68,7 +68,7 @@ static void init_send_garps(void);
 static void destroy_send_garps(void);
 static void send_garp_wait(void);
 static void send_garp_run(const struct ovsrec_bridge *,
-                          const char *chassis_id,
+                          const struct sbrec_chassis *,
                           const struct lport_index *lports,
                           struct hmap *local_datapaths);
 static void pinctrl_handle_nd_na(const struct flow *ip_flow,
@@ -663,7 +663,7 @@ process_packet_in(const struct ofp_header *msg)
 
     struct ofputil_packet_in pin;
     struct ofpbuf continuation;
-    enum ofperr error = ofputil_decode_packet_in(msg, true, &pin,
+    enum ofperr error = ofputil_decode_packet_in(msg, true, NULL, &pin,
                                                  NULL, NULL, &continuation);
 
     if (error) {
@@ -729,8 +729,7 @@ pinctrl_recv(const struct ofp_header *oh, enum ofptype type)
     if (type == OFPTYPE_ECHO_REQUEST) {
         queue_msg(make_echo_reply(oh));
     } else if (type == OFPTYPE_GET_CONFIG_REPLY) {
-        /* Enable asynchronous messages (see "Asynchronous Messages" in
-         * DESIGN.md for more information). */
+        /* Enable asynchronous messages */
         struct ofputil_switch_config config;
 
         ofputil_decode_get_config_reply(oh, &config);
@@ -753,7 +752,7 @@ pinctrl_recv(const struct ofp_header *oh, enum ofptype type)
 void
 pinctrl_run(struct controller_ctx *ctx, const struct lport_index *lports,
             const struct ovsrec_bridge *br_int,
-            const char *chassis_id,
+            const struct sbrec_chassis *chassis,
             struct hmap *local_datapaths)
 {
     char *target = xasprintf("unix:%s/%s.mgmt", ovs_rundir(), br_int->name);
@@ -789,7 +788,7 @@ pinctrl_run(struct controller_ctx *ctx, const struct lport_index *lports,
     }
 
     run_put_mac_bindings(ctx, lports);
-    send_garp_run(br_int, chassis_id, lports, local_datapaths);
+    send_garp_run(br_int, chassis, lports, local_datapaths);
 }
 
 void
@@ -994,7 +993,6 @@ struct garp_data {
     long long int announce_time; /* Next announcement in ms. */
     int backoff;                 /* Backoff for the next announcement. */
     ofp_port_t ofport;           /* ofport used to output this GARP. */
-    int tag;                     /* VLAN tag of this GARP packet, or -1. */
 };
 
 /* Contains GARPs to be sent. */
@@ -1017,7 +1015,7 @@ destroy_send_garps(void)
 }
 
 static void
-add_garp(const char *name, ofp_port_t ofport, int tag,
+add_garp(const char *name, ofp_port_t ofport,
          const struct eth_addr ea, ovs_be32 ip)
 {
     struct garp_data *garp = xmalloc(sizeof *garp);
@@ -1026,7 +1024,6 @@ add_garp(const char *name, ofp_port_t ofport, int tag,
     garp->announce_time = time_msec() + 1000;
     garp->backoff = 1;
     garp->ofport = ofport;
-    garp->tag = tag;
     shash_add(&send_garp_data, name, garp);
 }
 
@@ -1045,7 +1042,6 @@ send_garp_update(const struct sbrec_port_binding *binding_rec,
     }
     ofp_port_t ofport = u16_to_ofp(simap_get(localnet_ofports,
                                              ld->localnet_port->logical_port));
-    int tag = ld->localnet_port->n_tag ? *ld->localnet_port->tag : -1;
 
     volatile struct garp_data *garp = NULL;
     /* Update GARP for NAT IP if it exists. */
@@ -1062,10 +1058,8 @@ send_garp_update(const struct sbrec_port_binding *binding_rec,
             garp = shash_find_data(&send_garp_data, name);
             if (garp) {
                 garp->ofport = ofport;
-                garp->tag = tag;
             } else {
-                add_garp(name, ofport, tag, laddrs->ea,
-                         laddrs->ipv4_addrs[i].addr);
+                add_garp(name, ofport, laddrs->ea, laddrs->ipv4_addrs[i].addr);
             }
             free(name);
         }
@@ -1088,7 +1082,7 @@ send_garp_update(const struct sbrec_port_binding *binding_rec,
             continue;
         }
 
-        add_garp(binding_rec->logical_port, ofport, tag,
+        add_garp(binding_rec->logical_port, ofport,
                  laddrs.ea, laddrs.ipv4_addrs[0].addr);
 
         destroy_lport_addresses(&laddrs);
@@ -1118,11 +1112,6 @@ send_garp(struct garp_data *garp, long long int current_time)
     compose_arp(&packet, ARP_OP_REQUEST, garp->ea, eth_addr_zero,
                 true, garp->ipv4, garp->ipv4);
 
-    /* Compose a GARP request packet's vlan if exist. */
-    if (garp->tag >= 0) {
-        eth_push_vlan(&packet, htons(ETH_TYPE_VLAN), htons(garp->tag));
-    }
-
     /* Compose actions.  The garp request is output on localnet ofport. */
     uint64_t ofpacts_stub[4096 / 8];
     struct ofpbuf ofpacts = OFPBUF_STUB_INITIALIZER(ofpacts_stub);
@@ -1156,7 +1145,7 @@ send_garp(struct garp_data *garp, long long int current_time)
 /* Get localnet vifs, local l3gw ports and ofport for localnet patch ports. */
 static void
 get_localnet_vifs_l3gwports(const struct ovsrec_bridge *br_int,
-                  const char *this_chassis_id,
+                  const struct sbrec_chassis *chassis,
                   const struct lport_index *lports,
                   struct hmap *local_datapaths,
                   struct sset *localnet_vifs,
@@ -1170,13 +1159,11 @@ get_localnet_vifs_l3gwports(const struct ovsrec_bridge *br_int,
         }
         const char *chassis_id = smap_get(&port_rec->external_ids,
                                           "ovn-chassis-id");
-        if (chassis_id && !strcmp(chassis_id, this_chassis_id)) {
+        if (chassis_id && !strcmp(chassis_id, chassis->name)) {
             continue;
         }
         const char *localnet = smap_get(&port_rec->external_ids,
                                         "ovn-localnet-port");
-        const char *l3_gateway_port = smap_get(&port_rec->external_ids,
-                                               "ovn-l3gateway-port");
         for (int j = 0; j < port_rec->n_interfaces; j++) {
             const struct ovsrec_interface *iface_rec = port_rec->interfaces[j];
             if (!iface_rec->n_ofport) {
@@ -1190,10 +1177,6 @@ get_localnet_vifs_l3gwports(const struct ovsrec_bridge *br_int,
                 simap_put(localnet_ofports, localnet, ofport);
                 continue;
             }
-            if (l3_gateway_port) {
-                sset_add(local_l3gw_ports, l3_gateway_port);
-                continue;
-            }
             const char *iface_id = smap_get(&iface_rec->external_ids,
                                             "iface-id");
             if (!iface_id) {
@@ -1212,6 +1195,21 @@ get_localnet_vifs_l3gwports(const struct ovsrec_bridge *br_int,
             }
         }
     }
+
+    const struct local_datapath *ld;
+    HMAP_FOR_EACH (ld, hmap_node, local_datapaths) {
+        if (!ld->has_local_l3gateway) {
+            continue;
+        }
+
+        for (size_t i = 0; i < ld->ldatapath->n_lports; i++) {
+            const struct sbrec_port_binding *pb = ld->ldatapath->lports[i];
+            if (!strcmp(pb->type, "l3gateway")
+                /* && it's on this chassis */) {
+                sset_add(local_l3gw_ports, pb->logical_port);
+            }
+        }
+    }
 }
 
 static void
@@ -1256,7 +1254,8 @@ send_garp_wait(void)
 }
 
 static void
-send_garp_run(const struct ovsrec_bridge *br_int, const char *chassis_id,
+send_garp_run(const struct ovsrec_bridge *br_int,
+              const struct sbrec_chassis *chassis,
               const struct lport_index *lports,
               struct hmap *local_datapaths)
 {
@@ -1268,7 +1267,7 @@ send_garp_run(const struct ovsrec_bridge *br_int, const char *chassis_id,
 
     shash_init(&nat_addresses);
 
-    get_localnet_vifs_l3gwports(br_int, chassis_id, lports, local_datapaths,
+    get_localnet_vifs_l3gwports(br_int, chassis, lports, local_datapaths,
                       &localnet_vifs, &localnet_ofports, &local_l3gw_ports);
 
     get_nat_addresses_and_keys(&nat_ip_keys, &local_l3gw_ports, lports,
