@@ -25,11 +25,11 @@ struct hmap;
 struct lport_index;
 struct ovsrec_bridge;
 struct controller_ctx;
+struct sbrec_chassis;
 
 void pinctrl_init(void);
 void pinctrl_run(struct controller_ctx *, const struct lport_index *,
-                 const struct ovsrec_bridge *,
-                 const char *chassis_id,
+                 const struct ovsrec_bridge *, const struct sbrec_chassis *,
                  struct hmap *local_datapaths);
 void pinctrl_wait(struct controller_ctx *);
 void pinctrl_destroy(void);
