@@ -551,7 +551,8 @@ encode_CT_NEXT(const struct ovnact_next *next,
 {
     struct ofpact_conntrack *ct = ofpact_put_CT(ofpacts);
     ct->recirc_table = ep->first_ptable + next->ltable;
-    ct->zone_src.field = mf_from_id(MFF_LOG_CT_ZONE);
+    ct->zone_src.field = ep->is_switch ? mf_from_id(MFF_LOG_CT_ZONE)
+                            : mf_from_id(MFF_LOG_DNAT_ZONE);
     ct->zone_src.ofs = 0;
     ct->zone_src.n_bits = 16;
     ofpact_finish(ofpacts, &ct->ofpact);
@@ -844,7 +845,6 @@ parse_ct_lb_action(struct action_context *ctx)
         while (!lexer_match(ctx->lexer, LEX_T_RPAREN)) {
             if (ctx->lexer->token.type != LEX_T_INTEGER
                 || mf_subvalue_width(&ctx->lexer->token.value) > 32) {
-                free(dsts);
                 lexer_syntax_error(ctx->lexer, "expecting IPv4 address");
                 return;
             }
@@ -1488,7 +1488,7 @@ encode_put_dhcpv4_option(const struct ovnact_dhcp_option *o,
             if (c[i].masked) {
                 plen = (uint8_t) ip_count_cidr_bits(c[i].mask.ipv4);
             }
-            opt_header[1] += (1 + DIV_ROUND_UP(plen, 8) + sizeof(ovs_be32));
+            opt_header[1] += (1 + (plen / 8) + sizeof(ovs_be32)) ;
         }
 
         /* Copied from RFC 3442. Please refer to this RFC for the format of
@@ -1513,7 +1513,7 @@ encode_put_dhcpv4_option(const struct ovnact_dhcp_option *o,
                 plen = ip_count_cidr_bits(c[i].mask.ipv4);
             }
             ofpbuf_put(ofpacts, &plen, 1);
-            ofpbuf_put(ofpacts, &c[i].value.ipv4, DIV_ROUND_UP(plen, 8));
+            ofpbuf_put(ofpacts, &c[i].value.ipv4, plen / 8);
             ofpbuf_put(ofpacts, &c[i + 1].value.ipv4,
                        sizeof(ovs_be32));
         }
@@ -1620,6 +1620,46 @@ free_PUT_DHCPV6_OPTS(struct ovnact_put_dhcp_opts *pdo)
 {
     free_put_dhcp_opts(pdo);
 }
+
+static void
+parse_SET_QUEUE(struct action_context *ctx)
+{
+    int queue_id;
+
+    if (!lexer_force_match(ctx->lexer, LEX_T_LPAREN)
+        || !lexer_get_int(ctx->lexer, &queue_id)
+        || !lexer_force_match(ctx->lexer, LEX_T_RPAREN)) {
+        return;
+    }
+
+    if (queue_id < QDISC_MIN_QUEUE_ID || queue_id > QDISC_MAX_QUEUE_ID) {
+        lexer_error(ctx->lexer, "Queue ID %d for set_queue is "
+                    "not in valid range %d to %d.",
+                    queue_id, QDISC_MIN_QUEUE_ID, QDISC_MAX_QUEUE_ID);
+        return;
+    }
+
+    ovnact_put_SET_QUEUE(ctx->ovnacts)->queue_id = queue_id;
+}
+
+static void
+format_SET_QUEUE(const struct ovnact_set_queue *set_queue, struct ds *s)
+{
+    ds_put_format(s, "set_queue(%d);", set_queue->queue_id);
+}
+
+static void
+encode_SET_QUEUE(const struct ovnact_set_queue *set_queue,
+                 const struct ovnact_encode_params *ep OVS_UNUSED,
+                 struct ofpbuf *ofpacts)
+{
+    ofpact_put_SET_QUEUE(ofpacts)->queue_id = set_queue->queue_id;
+}
+
+static void
+free_SET_QUEUE(struct ovnact_set_queue *a OVS_UNUSED)
+{
+}
 
 /* Parses an assignment or exchange or put_dhcp_opts action. */
 static void
@@ -1691,6 +1731,8 @@ parse_action(struct action_context *ctx)
         parse_get_mac_bind(ctx, 128, ovnact_put_GET_ND(ctx->ovnacts));
     } else if (lexer_match_id(ctx->lexer, "put_nd")) {
         parse_put_mac_bind(ctx, 128, ovnact_put_PUT_ND(ctx->ovnacts));
+    } else if (lexer_match_id(ctx->lexer, "set_queue")) {
+        parse_SET_QUEUE(ctx);
     } else {
         lexer_syntax_error(ctx->lexer, "expecting action");
     }
