@@ -175,7 +175,8 @@ parse_options(int argc, char *argv[])
         OPT_TIMESTAMP,
         VLOG_OPTION_ENUMS,
         DAEMON_OPTION_ENUMS,
-        TABLE_OPTION_ENUMS
+        TABLE_OPTION_ENUMS,
+        SSL_OPTION_ENUMS,
     };
     static const struct option long_options[] = {
         {"help", no_argument, NULL, 'h'},
@@ -870,7 +871,7 @@ ovsdb_client_cond_change(struct unixctl_conn *conn, int argc OVS_UNUSED,
     jsonrpc_send(rpc, request);
 
     VLOG_DBG("cond change %s %s", argv[1], argv[2]);
-    unixctl_command_reply(conn, "condition changed");
+    unixctl_command_reply(conn, "condiiton changed");
 }
 
 static void
@@ -1299,9 +1300,9 @@ do_dump(struct jsonrpc *rpc, const char *database,
         for (i = 1; i < argc; i++) {
             node = shash_find(&tschema->columns, argv[i]);
             if (!node) {
-                ovs_fatal(0, "Table \"%s\" has no column %s.", argv[0], argv[i]);
+                ovs_fatal(0, "Table \"%s\" has no column %s.", argv[0], argv[1]);
             }
-            shash_add(&custom_columns, argv[i], node->data);
+            shash_add(&custom_columns, argv[1], node->data);
         }
     } else {
         tables = shash_sort(&schema->tables);
